package baseui;

/**
 * <p>Title: Gestion Affichage et processus</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * @author: Jean-Philippe Rennard
 * @version 1.0
 */

import java.awt.*;

/**  Classe abstraite de gestion graphique et animation */
public abstract class Cvue extends Canvas {
  /** Types de reprsentation **/
  public static final float HAUTEURL_DEFAUT=1000;
  public static final float LARGEURL_DEFAUT=1000;
  public static final int HAUTEURR_DEFAUT=1000;
  public static final int LARGEURR_DEFAUT=1000;
  public static final int REPRANISOTROPIC=0;
  public static final int REPRISOTROPIC=1;
  public static final int CADREX_DEFAUT=0;
  public static final int CADREY_DEFAUT=0;

  /** Type de reprsentation courant **/
  int reprCrt;
  /** Taille logique */
  public float lLargeur,lHauteur;
  /** Taille du Canvas = taille physique */
  public int rHauteur,rLargeur;
  /** Taille de l'encadrement (bordure de la zone s'affichage) */
  int cadreX, cadreY;
  /** Taille de la zone d'affichage = relle-cadre */
  public int aHauteur, aLargeur;
  /** origine des coordonnes */
  float origX, origY;
  /** Taille d'un pixel selon rapport rel/logique */
  float pixLargeur, pixHauteur;
  /** Image pour double buffering */
  Image imgBuff;
  /** Contexte graphique du double buffering */
  public Graphics grBuff;

  /** Constructeur par dfaut */
  public Cvue() {
  }

  /** Constructeur avec couleurs */
  public Cvue(Color c, Color f, int repr) {
    reprCrt=repr;
    setDim(LARGEURL_DEFAUT, HAUTEURL_DEFAUT,
                LARGEURR_DEFAUT, HAUTEURR_DEFAUT,
                CADREX_DEFAUT, CADREY_DEFAUT,0,0);
    redimCadre(0,0);
    setBackground(f);
    setForeground(c);
  }
  /** Mise  jour des dimensions */
  public void setDim(float ll, float hl, int lr, int hr,
                         int cx, int cy, float ox, float oy) {
    lLargeur=ll;
    lHauteur=hl;
    origX=ox;
    origY=oy;
    redimCadre(cx, cy);
  }
  /** Redimension du cadre */
  public void redimCadre(int cx, int cy) {
    cadreX=cx;
    cadreY=cy;
    reDim(rLargeur, rHauteur);
  }
  /** Mise  jour des valeurs de travail **/
  public void reDim(int mx,int my) {
    rLargeur=mx;
    aLargeur=rLargeur-2*cadreX;
    rHauteur=my;
    aHauteur=rHauteur-2*cadreY;
    setLogicSize(lLargeur, lHauteur);
  }
  /** Mise  jour de la taille logique **/
  void setLogicSize(float w,float h) {
    lLargeur=w;
    lHauteur=h;
    switch(reprCrt) {
      case REPRANISOTROPIC:
        pixLargeur=lLargeur/aLargeur;
        pixHauteur=lHauteur/aHauteur;
        break;
      case REPRISOTROPIC:
        if(lLargeur/aLargeur<lHauteur/aHauteur) {
          aLargeur=aHauteur;
          pixLargeur=pixHauteur=lHauteur/aHauteur;
          origX=rLongueurToLX( (int)((rLargeur-aLargeur)/2.0) );
          origY=0;
        }
        else {
          aHauteur=aLargeur;
          pixLargeur=pixHauteur=lLargeur/aLargeur;
          origY=rLongueurToLY( (int)((rHauteur-aHauteur)/2.0) );
          origX=0;
        }
        break;
    }
  }
  /** mise  jour des dimensions selon canvas. Renvoit dimension */
  Dimension setDimCanvas() {
    Dimension d=getParent().getSize();
    reDim(d.width-1, d.height-1);
    return d;
  }

  /** Modification du type de reprsentation **/
  public void setRepresentation(int r) {
    switch(r) {
      case REPRISOTROPIC:
        reprCrt=r;
        break;
      case REPRANISOTROPIC:
      default :
        reprCrt=REPRANISOTROPIC;
    }
    setLogicSize(lLargeur, lHauteur);
  }
  /** Renvoit le mode de reprsentation actif **/
  public int getRepresentation() {
    return reprCrt;
  }

  /** Conversion logique vers rel en X */
  public int   lToRX(float _x) {
    return Math.round((_x+origX)/pixLargeur)+cadreX;
  }
  /** Conversion logique vers rel en Y */
  public int lToRY(float _y) {
    return Math.round((_y+origY)/pixHauteur)+cadreY;
  }
  /** Conversion rel vers logique en X */
  public float rToLX(int _x) {
    return (_x-cadreX)*pixLargeur-origX;
  }
  /** Conversion rel vers logique en Y */
  public float rToLY(int _y) {
    return (_y-cadreY)*pixHauteur-origY;
  }
  /** Transforme une longueur logique en rel en X */
  public int lLongueurToRX(float l) {
    return Math.round(l/pixLargeur);
  }
  /** Transforme une longueur logique en rel en Y */
  public int lLongueurToRY(float l) {
    return Math.round(l/pixHauteur);
  }
  /** Transforme une longueur relle en logique en X */
  public float rLongueurToLX(int l) {
    return l*pixLargeur;
  }
  /** Transforme une longueur relle en logique en Y */
  public float rLongueurToLY(int l) {
    return l*pixHauteur;
  }

//===================== ANIMATIONS

  /** Initialisation/Mise  jour de la taille de la vue */
  public void redimGrBuff() {
    setDimCanvas();
    imgBuff=null;
    imgBuff=createImage(rLargeur,rHauteur);
    grBuff=imgBuff.getGraphics();
  }
  /** Efface le fond */
  public void effaceFond() {
    grBuff.setColor(getBackground());
    grBuff.fillRect(0,0,rLargeur,rHauteur);
    grBuff.setColor(this.getForeground());
  }

  public void repaint() {
    paint(this.getGraphics());
  }
  public void paint(Graphics g) {
    if(g!=null)
      g.drawImage(imgBuff,0,0,this);
  }
  /** destruction du buffer */
  public void destroy() {
    grBuff.dispose();
  }
}
