package nimapplet;

/**
 * Package nimapplet
 * Jeu de Nim contre un rseau de McCulloch-Pitts.
 * Rseaux Neauronaux, Vuibert 2006.
 * Jean-Philippe Rennard
 * version 1.0, 17/3/2006
 */

import java.awt.*;
import baseui.*;

/**
 * <p>Title: Processus Nim</p>
 * <p>Description: Gestion du processus du jeu de Nim.</p>
 */
public class NimProcess extends Cprocess {
  public static final int JOUEUR_ORDI=0;
  public static final int JOUEUR_JOUEUR=1;
  public static final int CASE_VIDE=0;
  public static final int CASE_ORDI=1;
  public static final int CASE_JOUEUR=2;

  public static final int NB_CASE = 5*4+2;

  /** Pointeur sur l'interface utilisateur */
  NimUI appUI;
  /** Vue */
  NimVue nimVue;
  /** Nombre de cases du jeu */
  int nbCases;
  /** Nombre de cases occupes */
  int nbCasesOccupe;
  /** Tableau des cases */
  int cases[];
  /** Joueur courant */
  int joueurCrt;
  /** Reseau de calcul */
  ReseauMPNim reseauNim;

  /**
   * Constructeur du processus dans une interface utilisateur.
   * @param ui NimUI : interface utilisateur
   */
  public NimProcess(NimUI ui) {
    super(ui);
    appUI=ui;
    nimVue=new NimVue(this, Color.white, Color.black);
    initJeu();
  }
  /** Initialisation du jeu */
  public void initJeu() {
    nbCases=NB_CASE;
    nbCasesOccupe=0;
    cases=new int[nbCases];
    for (int i=0;i<nbCases;i++)
      cases[i]=CASE_VIDE;
    joueurCrt=JOUEUR_JOUEUR;
    reseauNim=new ReseauMPNim();
  }

  /** Passe = change joueur */
  public void changeJoueur() {
    joueurCrt = joueurCrt==JOUEUR_ORDI ? JOUEUR_JOUEUR : JOUEUR_ORDI;
  }

  /** Joue un coup de n cases */
  public boolean joue(int n) {
    if(nbCasesOccupe+n>nbCases)
      n=nbCases-nbCasesOccupe;
    for(int i=0;i<n;i++) {
      cases[nbCasesOccupe+i]=
          joueurCrt==JOUEUR_JOUEUR ? CASE_JOUEUR : CASE_ORDI;
    }
    nbCasesOccupe += n;
    afficheCoup();
    return nbCasesOccupe==nbCases ? true : false;
  }

  /** Test coup */
  /**
   * Test coup
   * @param c int : valeur du coup (nombre de btons).
   * @return boolean : coup gagnant ?
   */
  boolean testeCoup(int c) {
    double ve[]=new double[2];
    String s1=Integer.toBinaryString(nbCasesOccupe&3);
    String s2=Integer.toBinaryString(c&3);
    int inp1[] = new int[6];
    int inp2[] = new int[6];
    for(int j=s1.length(); j>0;j--) {
      inp1[s1.length()-j]= s1.charAt(j-1) - '0';
    }
    for(int j=s2.length(); j>0;j--) {
      inp2[s2.length()-j]= s2.charAt(j-1) - '0';
    }
    return reseauNim.testeCoup(inp1, inp2);
  }

  /** Coup ordinateur */
  public boolean coupOrdi() {
    int coup=1;
    for(int i=1;i<4;i++) {
      if(testeCoup(i)) {
        coup=i;
        break;
      }
    }
    return joue(coup);
  }

  /** affichage du jeu */
  public void afficheCoup() {
    nimVue.affJeu();
  }
  /** renvoie texte d'info */
  public String getTexteInfo() {
    String s;
    s=Integer.toString(nbCasesOccupe) + " cases occupes. " +
        Integer.toString(nbCases-nbCasesOccupe) + " cases libres. ";
    switch(joueurCrt) {
        case JOUEUR_JOUEUR:
          s += "A vous de jouer.";
          break;
        default:
          s += "A moi de jouer.";
    }
    return s;
  }
  /** getter nombre de cases */
  public int getNbCases() {
    return nbCases;
  }
  /** Initialisation/MAJ de la vue */
  public void initVue() {
    nimVue.setNewSize();
    nimVue.affJeu();
  }

  /** Passe au joueur suivant */
  public void geneSuiv() {
    if(joueurCrt==JOUEUR_ORDI) {
      coupOrdi();
      joueurCrt=JOUEUR_JOUEUR;
    }
  }
}
