package nimapplet;

/**
 * Package nimapplet
 * Jeu de Nim contre un rseau de McCulloch-Pitts.
 * Rseaux Neauronaux, Vuibert 2006.
 * Jean-Philippe Rennard
 * version 1.0, 17/3/2006
 */

import java.awt.*;
import baseui.*;

/**
 * <p>Title: Vue de l'applet Nim</p>
 * <p>Description: Interface graphique de l'applet Nim.</p>
 */
public class NimVue extends Cvue {
  static final float LARGEUR_PLATEAU=800;
  static final float HAUTEUR_PLATEAU=600;
  static final float LARGEUR_CASE=LARGEUR_PLATEAU/(NimProcess.NB_CASE*2);//30;
  static final float HAUTEUR_CASE=500;

  /** Pointeur sur processus */
  NimProcess partieNim;

  /** Constructeur dans un processus */
  public NimVue(NimProcess m) {
    super(Color.black, Color.white, Cvue.REPRANISOTROPIC);
    partieNim = m;
  }

  /** Constructeur dans un processus avec couleurs */
  public NimVue(NimProcess m, Color c, Color f) {
    super(c, f, Cvue.REPRANISOTROPIC); //Cvue.REPRANISOTROPIC);
    partieNim = m;
  }

  /** Affichage jeu */
  public void affJeu() {
    affFond();
    for(int i=0; i<partieNim.nbCases; i++)
      affCase(i, partieNim.cases[i]);
      repaint();
  }

  /** Affichage du fond */
  public void affFond() {
    effaceFond();
    grBuff.drawRoundRect(lToRX((lLargeur-LARGEUR_PLATEAU)/2),
                         lToRY((lHauteur-HAUTEUR_PLATEAU)/2),
                         lLongueurToRX(LARGEUR_PLATEAU),
                         lLongueurToRY(HAUTEUR_PLATEAU),
                         lLongueurToRX(LARGEUR_PLATEAU/8),
                         lLongueurToRY(HAUTEUR_PLATEAU/4));
    for(int i=0; i<partieNim.getNbCases(); i++)
      affCase(i,NimProcess.CASE_VIDE);
  }

  /** Affichage d'une case selon rang et contenu */
  public void affCase(int rang, int type) {
   Color colcrt=grBuff.getColor();
   Color col;
   switch(type) {
     case NimProcess.CASE_VIDE:
       col=Color.white;
       break;
     case NimProcess.CASE_ORDI:
       col=Color.blue;
       break;
     case NimProcess.CASE_JOUEUR:
     default:
       col=Color.green;
   }
   grBuff.setColor(col);
   float posX=(lLargeur-LARGEUR_PLATEAU)/2+
       rang*LARGEUR_CASE+
       ((LARGEUR_PLATEAU-partieNim.getNbCases()*LARGEUR_CASE)/
        (partieNim.getNbCases()+1)) +
       rang*((LARGEUR_PLATEAU-partieNim.getNbCases()*LARGEUR_CASE)/
        (partieNim.getNbCases()+1));
   float posY=(lHauteur-HAUTEUR_PLATEAU)/2+
                              (HAUTEUR_PLATEAU-HAUTEUR_CASE)/2;
   grBuff.fillRoundRect(lToRX(posX), lToRY(posY),
                        lLongueurToRX(LARGEUR_CASE),
                        lLongueurToRY(HAUTEUR_CASE),
                        lLongueurToRX(LARGEUR_CASE),
                        lLongueurToRY(LARGEUR_CASE));
   grBuff.setColor(colcrt);
  }

  /** Changement de taille de la vue */
  public void setNewSize() {
    redimGrBuff();
    repaint();
  }

}
