package nimapplet;

/**
 * Package nimapplet
 * Jeu de Nim contre un rseau de McCulloch-Pitts.
 * Rseaux Neauronaux, Vuibert 2006.
 * Jean-Philippe Rennard
 * version 1.0, 17/3/2006
 */

import neuronespack.*;

/**
 * <p>Title: Rseau Nim</p>
 * <p>Description: Rseau de McCulloch-Pitts pour la gestion d'un jeu de Nim.</p>
 */
public class ReseauMPNim extends ReseauMP {

  /** Constructeur */
  public ReseauMPNim() {
    super(4);
    setDonneesCourantes(2, 1);
    setResultat(1);
    construitReseau();
  }

  /** Construction du rseau */
  public void construitReseau() {
    // Couches
    int paramReseau[] = {
        3, 2, 1, 2};
    double valSeuil[][] = {
        {
        1, 2, 3}
        , {
        1, 1, 0}
        , {
        1, 0, 0}
        , {
        1, 1, 0}
    };
    construitReseau(paramReseau, valSeuil);
    // Connexions entres
    boolean mc[][] = new boolean[2][3];
    Connexions.construitConnexionsCompletes(2, 3, mc);
    construitConnexionsEntrees(mc);
    // Connexions couches
    boolean mc2[][] = {
        {
        true, false}
        , {
        true, false}
        , {
        false, true}
    };
    construitConnexionsCouches(getCouche(0), getCouche(1), mc2);

    boolean mc6[][] = {
        {
        false, false, false}
        , {
        true, true, true}
        , {
        false, false, false}
    };
    construitConnexionsCouches(getCouche(0), getCouche(0), mc6);
    boolean mc3[][] = {
        {
        true}
        , {
        true}
    };
    construitConnexionsCouches((Couche) getLesCouches().elementAt(1),
                               (Couche) getLesCouches().elementAt(2), mc3);

    boolean mc4[][] = {
        {
        true, true}
    };
    construitConnexionsCouches((Couche) getLesCouches().elementAt(2),
                               (Couche) getLesCouches().elementAt(3), mc4);

    boolean mc7[][] = {
        {
        false, true}
        , {
        false, false}
    };
    construitConnexionsCouches((Couche) getLesCouches().elementAt(3),
                               (Couche) getLesCouches().elementAt(3), mc7);

    // Connexions sortie
    boolean mc5[][] = {
        {
        false}
        , {
        true}
    };
    construitConnexionsSorties(mc5);

    // poids des connexions
    double pdsConnect1[][] = {
        {
        ACTIF, 0}
        , {
        INHIBE, 0}
        , {
        0, ACTIF}
    };
    setPoidsConnexions( getConnexions(5), pdsConnect1);
    double pdsConnect4[][] = {
        {
        0, 0, 0}
        , {
        ACTIF, ACTIF, ACTIF}
        , {
        0, 0, 0}
    };
    setPoidsConnexions( (Connexions) getLesConnexions().elementAt(6), pdsConnect4);
    double pdsConnect2[][] = {
        {
        ACTIF}
        , {
        ACTIF}
    };
    setPoidsConnexions( (Connexions) getLesConnexions().elementAt(7), pdsConnect2);
    double pdsConnect3[][] = {
        {
        ACTIF, INHIBE}
    };
    setPoidsConnexions( (Connexions) getLesConnexions().elementAt(8), pdsConnect3);
    double pdsConnect5[][] = {
        {
        0, ACTIF}
        , {
        0, 0}
    };
    setPoidsConnexions( (Connexions) getLesConnexions().elementAt(9), pdsConnect5);
  }

  /** Test coup */
  boolean testeCoup(int inp1[], int inp2[]) {
    double valeurEntree[]=new double[2];
    int lesResultats[] = new int[6];
    for(int i=0; i<6;i++) {
      valeurEntree[0] = inp1[i];
      valeurEntree[1] = inp2[i];
      getDonneesCourantes().setVecteurEntree(valeurEntree);
      propagation();
      lesResultats[i] = (int)getResultats().getValeurSortie(0);
    }
    return lesResultats[5] == 0 ? false : true;
  }

}
